# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Session Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model
from ....core import StrEnum


class StatusEnum(StrEnum):
    AVAILABLE = "AVAILABLE"
    DS_ERROR = "DS_ERROR"
    ENDED = "ENDED"
    FAILED_TO_REQUEST = "FAILED_TO_REQUEST"


class ApimodelsUpdateGamesessionDSInformationRequest(Model):
    """Apimodels update gamesession DS information request (apimodels.UpdateGamesessionDSInformationRequest)

    Properties:
        created_region: (createdRegion) REQUIRED str

        deployment: (deployment) REQUIRED str

        description: (description) REQUIRED str

        ip: (ip) REQUIRED str

        port: (port) REQUIRED int

        region: (region) REQUIRED str

        server_id: (serverId) REQUIRED str

        source: (source) REQUIRED str

        status: (status) REQUIRED Union[str, StatusEnum]
    """

    # region fields

    created_region: str  # REQUIRED
    deployment: str  # REQUIRED
    description: str  # REQUIRED
    ip: str  # REQUIRED
    port: int  # REQUIRED
    region: str  # REQUIRED
    server_id: str  # REQUIRED
    source: str  # REQUIRED
    status: Union[str, StatusEnum]  # REQUIRED

    # endregion fields

    # region with_x methods

    def with_created_region(
        self, value: str
    ) -> ApimodelsUpdateGamesessionDSInformationRequest:
        self.created_region = value
        return self

    def with_deployment(
        self, value: str
    ) -> ApimodelsUpdateGamesessionDSInformationRequest:
        self.deployment = value
        return self

    def with_description(
        self, value: str
    ) -> ApimodelsUpdateGamesessionDSInformationRequest:
        self.description = value
        return self

    def with_ip(self, value: str) -> ApimodelsUpdateGamesessionDSInformationRequest:
        self.ip = value
        return self

    def with_port(self, value: int) -> ApimodelsUpdateGamesessionDSInformationRequest:
        self.port = value
        return self

    def with_region(self, value: str) -> ApimodelsUpdateGamesessionDSInformationRequest:
        self.region = value
        return self

    def with_server_id(
        self, value: str
    ) -> ApimodelsUpdateGamesessionDSInformationRequest:
        self.server_id = value
        return self

    def with_source(self, value: str) -> ApimodelsUpdateGamesessionDSInformationRequest:
        self.source = value
        return self

    def with_status(
        self, value: Union[str, StatusEnum]
    ) -> ApimodelsUpdateGamesessionDSInformationRequest:
        self.status = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "created_region"):
            result["createdRegion"] = str(self.created_region)
        elif include_empty:
            result["createdRegion"] = ""
        if hasattr(self, "deployment"):
            result["deployment"] = str(self.deployment)
        elif include_empty:
            result["deployment"] = ""
        if hasattr(self, "description"):
            result["description"] = str(self.description)
        elif include_empty:
            result["description"] = ""
        if hasattr(self, "ip"):
            result["ip"] = str(self.ip)
        elif include_empty:
            result["ip"] = ""
        if hasattr(self, "port"):
            result["port"] = int(self.port)
        elif include_empty:
            result["port"] = 0
        if hasattr(self, "region"):
            result["region"] = str(self.region)
        elif include_empty:
            result["region"] = ""
        if hasattr(self, "server_id"):
            result["serverId"] = str(self.server_id)
        elif include_empty:
            result["serverId"] = ""
        if hasattr(self, "source"):
            result["source"] = str(self.source)
        elif include_empty:
            result["source"] = ""
        if hasattr(self, "status"):
            result["status"] = str(self.status)
        elif include_empty:
            result["status"] = Union[str, StatusEnum]()
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        created_region: str,
        deployment: str,
        description: str,
        ip: str,
        port: int,
        region: str,
        server_id: str,
        source: str,
        status: Union[str, StatusEnum],
        **kwargs,
    ) -> ApimodelsUpdateGamesessionDSInformationRequest:
        instance = cls()
        instance.created_region = created_region
        instance.deployment = deployment
        instance.description = description
        instance.ip = ip
        instance.port = port
        instance.region = region
        instance.server_id = server_id
        instance.source = source
        instance.status = status
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ApimodelsUpdateGamesessionDSInformationRequest:
        instance = cls()
        if not dict_:
            return instance
        if "createdRegion" in dict_ and dict_["createdRegion"] is not None:
            instance.created_region = str(dict_["createdRegion"])
        elif include_empty:
            instance.created_region = ""
        if "deployment" in dict_ and dict_["deployment"] is not None:
            instance.deployment = str(dict_["deployment"])
        elif include_empty:
            instance.deployment = ""
        if "description" in dict_ and dict_["description"] is not None:
            instance.description = str(dict_["description"])
        elif include_empty:
            instance.description = ""
        if "ip" in dict_ and dict_["ip"] is not None:
            instance.ip = str(dict_["ip"])
        elif include_empty:
            instance.ip = ""
        if "port" in dict_ and dict_["port"] is not None:
            instance.port = int(dict_["port"])
        elif include_empty:
            instance.port = 0
        if "region" in dict_ and dict_["region"] is not None:
            instance.region = str(dict_["region"])
        elif include_empty:
            instance.region = ""
        if "serverId" in dict_ and dict_["serverId"] is not None:
            instance.server_id = str(dict_["serverId"])
        elif include_empty:
            instance.server_id = ""
        if "source" in dict_ and dict_["source"] is not None:
            instance.source = str(dict_["source"])
        elif include_empty:
            instance.source = ""
        if "status" in dict_ and dict_["status"] is not None:
            instance.status = str(dict_["status"])
        elif include_empty:
            instance.status = Union[str, StatusEnum]()
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ApimodelsUpdateGamesessionDSInformationRequest]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ApimodelsUpdateGamesessionDSInformationRequest]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ApimodelsUpdateGamesessionDSInformationRequest,
        List[ApimodelsUpdateGamesessionDSInformationRequest],
        Dict[Any, ApimodelsUpdateGamesessionDSInformationRequest],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "createdRegion": "created_region",
            "deployment": "deployment",
            "description": "description",
            "ip": "ip",
            "port": "port",
            "region": "region",
            "serverId": "server_id",
            "source": "source",
            "status": "status",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "createdRegion": True,
            "deployment": True,
            "description": True,
            "ip": True,
            "port": True,
            "region": True,
            "serverId": True,
            "source": True,
            "status": True,
        }

    @staticmethod
    def get_enum_map() -> Dict[str, List[Any]]:
        return {
            "status": ["AVAILABLE", "DS_ERROR", "ENDED", "FAILED_TO_REQUEST"],
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

# AccelByte Gaming Services Iam Service

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from .....core import Operation
from .....core import HeaderStr
from .....core import HttpResponse


class Verify2faCodeForward(Operation):
    """Verify 2FA code (Verify2FACodeForward)

    Verifies the 2FA code.

    This is a forward version for '/mfa/verify'. If there is any error, it will redirect to login website with error details.
    If success, it will forward to auth request redirect url.
    If got error, it will forward to login website.

    ## 2FA remember device
    To remember device for 2FA, the request should provide cookie: device_token or header: Device-Token

    Properties:
        url: /iam/v3/oauth/mfa/verify/forward

        method: POST

        tags: ["OAuth2.0"]

        consumes: ["application/x-www-form-urlencoded"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        default_factor: (defaultFactor) OPTIONAL str in form_data

        factors: (factors) OPTIONAL str in form_data

        remember_device: (rememberDevice) OPTIONAL bool in form_data

        client_id: (clientId) REQUIRED str in form_data

        code: (code) REQUIRED str in form_data

        factor: (factor) REQUIRED str in form_data

        mfa_token: (mfaToken) REQUIRED str in form_data

    Responses:
        302: Found - (Found. Redirect to clients redirection URL with code or redirect to login web with error on the query parameter)
    """

    # region fields

    _url: str = "/iam/v3/oauth/mfa/verify/forward"
    _method: str = "POST"
    _consumes: List[str] = ["application/x-www-form-urlencoded"]
    _produces: List[str] = ["application/json"]
    _securities: List[List[str]] = [["BEARER_AUTH"]]
    _location_query: str = None

    default_factor: str  # OPTIONAL in [form_data]
    factors: str  # OPTIONAL in [form_data]
    remember_device: bool  # OPTIONAL in [form_data]
    client_id: str  # REQUIRED in [form_data]
    code: str  # REQUIRED in [form_data]
    factor: str  # REQUIRED in [form_data]
    mfa_token: str  # REQUIRED in [form_data]

    # endregion fields

    # region properties

    @property
    def url(self) -> str:
        return self._url

    @property
    def method(self) -> str:
        return self._method

    @property
    def consumes(self) -> List[str]:
        return self._consumes

    @property
    def produces(self) -> List[str]:
        return self._produces

    @property
    def securities(self) -> List[List[str]]:
        return self._securities

    @property
    def location_query(self) -> str:
        return self._location_query

    # endregion properties

    # region get methods

    # endregion get methods

    # region get_x_params methods

    def get_all_params(self) -> dict:
        return {
            "form_data": self.get_form_data_params(),
        }

    def get_form_data_params(self) -> dict:
        result = {}
        if hasattr(self, "default_factor"):
            result["defaultFactor"] = self.default_factor
        if hasattr(self, "factors"):
            result["factors"] = self.factors
        if hasattr(self, "remember_device"):
            result["rememberDevice"] = self.remember_device
        if hasattr(self, "client_id"):
            result["clientId"] = self.client_id
        if hasattr(self, "code"):
            result["code"] = self.code
        if hasattr(self, "factor"):
            result["factor"] = self.factor
        if hasattr(self, "mfa_token"):
            result["mfaToken"] = self.mfa_token
        return result

    # endregion get_x_params methods

    # region is/has methods

    # noinspection PyMethodMayBeStatic
    def has_redirects(self) -> bool:
        """Returns True if this operation has redirects, otherwise False.

        302: Found - (Found. Redirect to clients redirection URL with code or redirect to login web with error on the query parameter)
        """
        return True

    # endregion is/has methods

    # region with_x methods

    def with_default_factor(self, value: str) -> Verify2faCodeForward:
        self.default_factor = value
        return self

    def with_factors(self, value: str) -> Verify2faCodeForward:
        self.factors = value
        return self

    def with_remember_device(self, value: bool) -> Verify2faCodeForward:
        self.remember_device = value
        return self

    def with_client_id(self, value: str) -> Verify2faCodeForward:
        self.client_id = value
        return self

    def with_code(self, value: str) -> Verify2faCodeForward:
        self.code = value
        return self

    def with_factor(self, value: str) -> Verify2faCodeForward:
        self.factor = value
        return self

    def with_mfa_token(self, value: str) -> Verify2faCodeForward:
        self.mfa_token = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "default_factor") and self.default_factor:
            result["defaultFactor"] = str(self.default_factor)
        elif include_empty:
            result["defaultFactor"] = ""
        if hasattr(self, "factors") and self.factors:
            result["factors"] = str(self.factors)
        elif include_empty:
            result["factors"] = ""
        if hasattr(self, "remember_device") and self.remember_device:
            result["rememberDevice"] = bool(self.remember_device)
        elif include_empty:
            result["rememberDevice"] = False
        if hasattr(self, "client_id") and self.client_id:
            result["clientId"] = str(self.client_id)
        elif include_empty:
            result["clientId"] = ""
        if hasattr(self, "code") and self.code:
            result["code"] = str(self.code)
        elif include_empty:
            result["code"] = ""
        if hasattr(self, "factor") and self.factor:
            result["factor"] = str(self.factor)
        elif include_empty:
            result["factor"] = ""
        if hasattr(self, "mfa_token") and self.mfa_token:
            result["mfaToken"] = str(self.mfa_token)
        elif include_empty:
            result["mfaToken"] = ""
        return result

    # endregion to methods

    # region response methods

    # noinspection PyMethodMayBeStatic
    def parse_response(
        self, code: int, content_type: str, content: Any
    ) -> Tuple[Union[None, HttpResponse], Union[None, HttpResponse]]:
        """Parse the given response.

        302: Found - (Found. Redirect to clients redirection URL with code or redirect to login web with error on the query parameter)

        ---: HttpResponse (Undocumented Response)

        ---: HttpResponse (Unexpected Content-Type Error)

        ---: HttpResponse (Unhandled Error)
        """
        pre_processed_response, error = self.pre_process_response(
            code=code, content_type=content_type, content=content
        )
        if error is not None:
            return None, None if error.is_no_content() else error
        code, content_type, content = pre_processed_response

        if code == 302:
            return HttpResponse.create_redirect(code, content), None

        return self.handle_undocumented_response(
            code=code, content_type=content_type, content=content
        )

    # endregion response methods

    # region static methods

    @classmethod
    def create(
        cls,
        client_id: str,
        code: str,
        factor: str,
        mfa_token: str,
        default_factor: Optional[str] = None,
        factors: Optional[str] = None,
        remember_device: Optional[bool] = None,
        **kwargs,
    ) -> Verify2faCodeForward:
        instance = cls()
        instance.client_id = client_id
        instance.code = code
        instance.factor = factor
        instance.mfa_token = mfa_token
        if default_factor is not None:
            instance.default_factor = default_factor
        if factors is not None:
            instance.factors = factors
        if remember_device is not None:
            instance.remember_device = remember_device
        if x_flight_id := kwargs.get("x_flight_id", None):
            instance.x_flight_id = x_flight_id
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Verify2faCodeForward:
        instance = cls()
        if "defaultFactor" in dict_ and dict_["defaultFactor"] is not None:
            instance.default_factor = str(dict_["defaultFactor"])
        elif include_empty:
            instance.default_factor = ""
        if "factors" in dict_ and dict_["factors"] is not None:
            instance.factors = str(dict_["factors"])
        elif include_empty:
            instance.factors = ""
        if "rememberDevice" in dict_ and dict_["rememberDevice"] is not None:
            instance.remember_device = bool(dict_["rememberDevice"])
        elif include_empty:
            instance.remember_device = False
        if "clientId" in dict_ and dict_["clientId"] is not None:
            instance.client_id = str(dict_["clientId"])
        elif include_empty:
            instance.client_id = ""
        if "code" in dict_ and dict_["code"] is not None:
            instance.code = str(dict_["code"])
        elif include_empty:
            instance.code = ""
        if "factor" in dict_ and dict_["factor"] is not None:
            instance.factor = str(dict_["factor"])
        elif include_empty:
            instance.factor = ""
        if "mfaToken" in dict_ and dict_["mfaToken"] is not None:
            instance.mfa_token = str(dict_["mfaToken"])
        elif include_empty:
            instance.mfa_token = ""
        return instance

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "defaultFactor": "default_factor",
            "factors": "factors",
            "rememberDevice": "remember_device",
            "clientId": "client_id",
            "code": "code",
            "factor": "factor",
            "mfaToken": "mfa_token",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "defaultFactor": False,
            "factors": False,
            "rememberDevice": False,
            "clientId": True,
            "code": True,
            "factor": True,
            "mfaToken": True,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Chat Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model


class ModelsMessageResultWithAttributes(Model):
    """Models message result with attributes (models.MessageResultWithAttributes)

    Properties:
        action: (action) REQUIRED str

        final_message: (finalMessage) REQUIRED str

        id_: (id) REQUIRED str

        timestamp: (timestamp) REQUIRED int

        attributes: (attributes) OPTIONAL Dict[str, Any]

        cencored_words: (cencoredWords) OPTIONAL List[str]

        classifications: (classifications) OPTIONAL List[str]

        reference_id: (referenceId) OPTIONAL str
    """

    # region fields

    action: str  # REQUIRED
    final_message: str  # REQUIRED
    id_: str  # REQUIRED
    timestamp: int  # REQUIRED
    attributes: Dict[str, Any]  # OPTIONAL
    cencored_words: List[str]  # OPTIONAL
    classifications: List[str]  # OPTIONAL
    reference_id: str  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_action(self, value: str) -> ModelsMessageResultWithAttributes:
        self.action = value
        return self

    def with_final_message(self, value: str) -> ModelsMessageResultWithAttributes:
        self.final_message = value
        return self

    def with_id(self, value: str) -> ModelsMessageResultWithAttributes:
        self.id_ = value
        return self

    def with_timestamp(self, value: int) -> ModelsMessageResultWithAttributes:
        self.timestamp = value
        return self

    def with_attributes(
        self, value: Dict[str, Any]
    ) -> ModelsMessageResultWithAttributes:
        self.attributes = value
        return self

    def with_cencored_words(
        self, value: List[str]
    ) -> ModelsMessageResultWithAttributes:
        self.cencored_words = value
        return self

    def with_classifications(
        self, value: List[str]
    ) -> ModelsMessageResultWithAttributes:
        self.classifications = value
        return self

    def with_reference_id(self, value: str) -> ModelsMessageResultWithAttributes:
        self.reference_id = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "action"):
            result["action"] = str(self.action)
        elif include_empty:
            result["action"] = ""
        if hasattr(self, "final_message"):
            result["finalMessage"] = str(self.final_message)
        elif include_empty:
            result["finalMessage"] = ""
        if hasattr(self, "id_"):
            result["id"] = str(self.id_)
        elif include_empty:
            result["id"] = ""
        if hasattr(self, "timestamp"):
            result["timestamp"] = int(self.timestamp)
        elif include_empty:
            result["timestamp"] = 0
        if hasattr(self, "attributes"):
            result["attributes"] = {str(k0): v0 for k0, v0 in self.attributes.items()}
        elif include_empty:
            result["attributes"] = {}
        if hasattr(self, "cencored_words"):
            result["cencoredWords"] = [str(i0) for i0 in self.cencored_words]
        elif include_empty:
            result["cencoredWords"] = []
        if hasattr(self, "classifications"):
            result["classifications"] = [str(i0) for i0 in self.classifications]
        elif include_empty:
            result["classifications"] = []
        if hasattr(self, "reference_id"):
            result["referenceId"] = str(self.reference_id)
        elif include_empty:
            result["referenceId"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        action: str,
        final_message: str,
        id_: str,
        timestamp: int,
        attributes: Optional[Dict[str, Any]] = None,
        cencored_words: Optional[List[str]] = None,
        classifications: Optional[List[str]] = None,
        reference_id: Optional[str] = None,
        **kwargs,
    ) -> ModelsMessageResultWithAttributes:
        instance = cls()
        instance.action = action
        instance.final_message = final_message
        instance.id_ = id_
        instance.timestamp = timestamp
        if attributes is not None:
            instance.attributes = attributes
        if cencored_words is not None:
            instance.cencored_words = cencored_words
        if classifications is not None:
            instance.classifications = classifications
        if reference_id is not None:
            instance.reference_id = reference_id
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ModelsMessageResultWithAttributes:
        instance = cls()
        if not dict_:
            return instance
        if "action" in dict_ and dict_["action"] is not None:
            instance.action = str(dict_["action"])
        elif include_empty:
            instance.action = ""
        if "finalMessage" in dict_ and dict_["finalMessage"] is not None:
            instance.final_message = str(dict_["finalMessage"])
        elif include_empty:
            instance.final_message = ""
        if "id" in dict_ and dict_["id"] is not None:
            instance.id_ = str(dict_["id"])
        elif include_empty:
            instance.id_ = ""
        if "timestamp" in dict_ and dict_["timestamp"] is not None:
            instance.timestamp = int(dict_["timestamp"])
        elif include_empty:
            instance.timestamp = 0
        if "attributes" in dict_ and dict_["attributes"] is not None:
            instance.attributes = {
                str(k0): v0 for k0, v0 in dict_["attributes"].items()
            }
        elif include_empty:
            instance.attributes = {}
        if "cencoredWords" in dict_ and dict_["cencoredWords"] is not None:
            instance.cencored_words = [str(i0) for i0 in dict_["cencoredWords"]]
        elif include_empty:
            instance.cencored_words = []
        if "classifications" in dict_ and dict_["classifications"] is not None:
            instance.classifications = [str(i0) for i0 in dict_["classifications"]]
        elif include_empty:
            instance.classifications = []
        if "referenceId" in dict_ and dict_["referenceId"] is not None:
            instance.reference_id = str(dict_["referenceId"])
        elif include_empty:
            instance.reference_id = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ModelsMessageResultWithAttributes]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ModelsMessageResultWithAttributes]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ModelsMessageResultWithAttributes,
        List[ModelsMessageResultWithAttributes],
        Dict[Any, ModelsMessageResultWithAttributes],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "action": "action",
            "finalMessage": "final_message",
            "id": "id_",
            "timestamp": "timestamp",
            "attributes": "attributes",
            "cencoredWords": "cencored_words",
            "classifications": "classifications",
            "referenceId": "reference_id",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "action": True,
            "finalMessage": True,
            "id": True,
            "timestamp": True,
            "attributes": False,
            "cencoredWords": False,
            "classifications": False,
            "referenceId": False,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Group Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model

from ..models.models_group_rule_response_v1_group_custom_rule import (
    ModelsGroupRuleResponseV1GroupCustomRule,
)
from ..models.models_rule_response_v1 import ModelsRuleResponseV1


class ModelsGroupRuleResponseV1(Model):
    """Models group rule response V1 (models.GroupRuleResponseV1)

    Properties:
        group_custom_rule: (groupCustomRule) REQUIRED ModelsGroupRuleResponseV1GroupCustomRule

        group_predefined_rules: (groupPredefinedRules) REQUIRED List[ModelsRuleResponseV1]
    """

    # region fields

    group_custom_rule: ModelsGroupRuleResponseV1GroupCustomRule  # REQUIRED
    group_predefined_rules: List[ModelsRuleResponseV1]  # REQUIRED

    # endregion fields

    # region with_x methods

    def with_group_custom_rule(
        self, value: ModelsGroupRuleResponseV1GroupCustomRule
    ) -> ModelsGroupRuleResponseV1:
        self.group_custom_rule = value
        return self

    def with_group_predefined_rules(
        self, value: List[ModelsRuleResponseV1]
    ) -> ModelsGroupRuleResponseV1:
        self.group_predefined_rules = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "group_custom_rule"):
            result["groupCustomRule"] = self.group_custom_rule.to_dict(
                include_empty=include_empty
            )
        elif include_empty:
            result["groupCustomRule"] = ModelsGroupRuleResponseV1GroupCustomRule()
        if hasattr(self, "group_predefined_rules"):
            result["groupPredefinedRules"] = [
                i0.to_dict(include_empty=include_empty)
                for i0 in self.group_predefined_rules
            ]
        elif include_empty:
            result["groupPredefinedRules"] = []
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        group_custom_rule: ModelsGroupRuleResponseV1GroupCustomRule,
        group_predefined_rules: List[ModelsRuleResponseV1],
        **kwargs,
    ) -> ModelsGroupRuleResponseV1:
        instance = cls()
        instance.group_custom_rule = group_custom_rule
        instance.group_predefined_rules = group_predefined_rules
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ModelsGroupRuleResponseV1:
        instance = cls()
        if not dict_:
            return instance
        if "groupCustomRule" in dict_ and dict_["groupCustomRule"] is not None:
            instance.group_custom_rule = (
                ModelsGroupRuleResponseV1GroupCustomRule.create_from_dict(
                    dict_["groupCustomRule"], include_empty=include_empty
                )
            )
        elif include_empty:
            instance.group_custom_rule = ModelsGroupRuleResponseV1GroupCustomRule()
        if (
            "groupPredefinedRules" in dict_
            and dict_["groupPredefinedRules"] is not None
        ):
            instance.group_predefined_rules = [
                ModelsRuleResponseV1.create_from_dict(i0, include_empty=include_empty)
                for i0 in dict_["groupPredefinedRules"]
            ]
        elif include_empty:
            instance.group_predefined_rules = []
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ModelsGroupRuleResponseV1]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ModelsGroupRuleResponseV1]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ModelsGroupRuleResponseV1,
        List[ModelsGroupRuleResponseV1],
        Dict[Any, ModelsGroupRuleResponseV1],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "groupCustomRule": "group_custom_rule",
            "groupPredefinedRules": "group_predefined_rules",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "groupCustomRule": True,
            "groupPredefinedRules": True,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import HeaderStr
from ....core import get_namespace as get_services_namespace
from ....core import run_request
from ....core import run_request_async
from ....core import same_doc_as

from ..models import IamErrorResponse
from ..models import ModelChallengeResponse
from ..models import ModelCreateChallengeRequest
from ..models import ModelListChallengeResponse
from ..models import ModelListPeriodsResponse
from ..models import ModelSchedule
from ..models import ModelUpdateChallengeRequest
from ..models import ModelUpdateChallengeScheduleRequest
from ..models import ResponseError

from ..operations.challenge_configuration import AdminCreateChallenge
from ..operations.challenge_configuration import AdminDeleteChallenge
from ..operations.challenge_configuration import AdminDeleteTiedChallenge
from ..operations.challenge_configuration import AdminGetActiveChallenges
from ..operations.challenge_configuration import AdminGetChallenge
from ..operations.challenge_configuration import AdminGetChallenges
from ..operations.challenge_configuration import (
    AdminGetChallengesSortByEnum,
    AdminGetChallengesStatusEnum,
)
from ..operations.challenge_configuration import AdminGetPeriods
from ..operations.challenge_configuration import AdminRandomizeChallenge
from ..operations.challenge_configuration import AdminUpdateChallenge
from ..operations.challenge_configuration import AdminUpdateTiedChallengeSchedule
from ..models import (
    ModelChallengeResponseAssignmentRuleEnum,
    ModelChallengeResponseGoalsVisibilityEnum,
    ModelChallengeResponseRotationEnum,
    ModelChallengeResponseStatusEnum,
)
from ..models import (
    ModelCreateChallengeRequestAssignmentRuleEnum,
    ModelCreateChallengeRequestGoalsVisibilityEnum,
    ModelCreateChallengeRequestRotationEnum,
)
from ..models import (
    ModelUpdateChallengeRequestAssignmentRuleEnum,
    ModelUpdateChallengeRequestGoalsVisibilityEnum,
    ModelUpdateChallengeRequestRotationEnum,
)
from ..models import ModelUpdateChallengeScheduleRequestActionEnum


@same_doc_as(AdminCreateChallenge)
def admin_create_challenge(
    body: ModelCreateChallengeRequest,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create New Challenge (adminCreateChallenge)

    - Required permission: ADMIN:NAMESPACE:{namespace}:CHALLENGE [CREATE]
    Challenge is a collection of goals that can be completed by players. Challenge can have rules that specify what and when goals will be available for players to be taken.
    #### Request body:
    - code: only lowercase letters, numbers, and the separator - are allowed; must start and end with letter
    - name: name of the challenge
    - description: text describing about the challenge (optional)
    - startDate: timestamp of when the challenge is started
    - endDate: timestamp of when the challenge is ended (optional)
    - endAfter: describe number of period challenge will be retired after (optional). To configure challenge that never end, leave the endDate and endAfter field null/empty.
    - repeatAfter: number of periods after which the challenge's goals repeat. Leave empty if you don't want to repeat the challenge.
    - note: when repeatAfter is set, goals cannot include the 'schedule' field when created via POST .../goals. Use PUT .../goals/{code}/slots to assign each goal to a specific slot instead.
    - rotation: describe how long goals in a challenge will be available for players to progress before rotated with another goals. (DAILY|WEEKLY|MONTHLY|NONE)
    - activeGoalsPerRotation: number of goals per rotation (currently only applicable for RANDOMIZE assignment)
    - assignmentRule: describe how the goals will be assigned and scheduled to users. (FIXED|RANDOMIZED|UNSCHEDULED|CUSTOM)
    - goalsVisibility: describe whether users can see all goals under challenge, or only active goal in one rotation period only. (SHOWALL|PERIODONLY)
    - resetConfig: describe when rotation reset will happen (optional).
    - resetTime: Reset time must follow hours:minutes in 24 hours format (e.g. 01:30, 23:15) and in UTC timezone. Default to "00:00"
    - resetDay: Reset Day follows the ISO-8601 standard, from 1 (Monday) to 7 (Sunday). Default to 1 in WEEKLY rotation.
    - resetDate: Reset Date must be a number 1 - 31. Default to 1 in MONTHLY rotation.
    - randomizedPerRotation:
    - true: each goal will be randomly assigned to multiple periods
    - false: a goal will only be assigned to one period
    - tags: challenge's labels.

    Properties:
        url: /challenge/v1/admin/namespaces/{namespace}/challenges

        method: POST

        tags: ["Challenge Configuration"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelCreateChallengeRequest in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        201: Created - ModelChallengeResponse (Created)

        400: Bad Request - ResponseError (20018: bad request: {{message}})

        401: Unauthorized - IamErrorResponse (20001: unauthorized access)

        403: Forbidden - IamErrorResponse (20013: insufficient permission)

        409: Conflict - ResponseError (99002: duplicate key error: {{message}})

        422: Unprocessable Entity - ResponseError (99003: challenge validation error: {{message}})

        500: Internal Server Error - ResponseError (20000: internal server error: {{message}})
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminCreateChallenge.create(
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminCreateChallenge)
async def admin_create_challenge_async(
    body: ModelCreateChallengeRequest,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create New Challenge (adminCreateChallenge)

    - Required permission: ADMIN:NAMESPACE:{namespace}:CHALLENGE [CREATE]
    Challenge is a collection of goals that can be completed by players. Challenge can have rules that specify what and when goals will be available for players to be taken.
    #### Request body:
    - code: only lowercase letters, numbers, and the separator - are allowed; must start and end with letter
    - name: name of the challenge
    - description: text describing about the challenge (optional)
    - startDate: timestamp of when the challenge is started
    - endDate: timestamp of when the challenge is ended (optional)
    - endAfter: describe number of period challenge will be retired after (optional). To configure challenge that never end, leave the endDate and endAfter field null/empty.
    - repeatAfter: number of periods after which the challenge's goals repeat. Leave empty if you don't want to repeat the challenge.
    - note: when repeatAfter is set, goals cannot include the 'schedule' field when created via POST .../goals. Use PUT .../goals/{code}/slots to assign each goal to a specific slot instead.
    - rotation: describe how long goals in a challenge will be available for players to progress before rotated with another goals. (DAILY|WEEKLY|MONTHLY|NONE)
    - activeGoalsPerRotation: number of goals per rotation (currently only applicable for RANDOMIZE assignment)
    - assignmentRule: describe how the goals will be assigned and scheduled to users. (FIXED|RANDOMIZED|UNSCHEDULED|CUSTOM)
    - goalsVisibility: describe whether users can see all goals under challenge, or only active goal in one rotation period only. (SHOWALL|PERIODONLY)
    - resetConfig: describe when rotation reset will happen (optional).
    - resetTime: Reset time must follow hours:minutes in 24 hours format (e.g. 01:30, 23:15) and in UTC timezone. Default to "00:00"
    - resetDay: Reset Day follows the ISO-8601 standard, from 1 (Monday) to 7 (Sunday). Default to 1 in WEEKLY rotation.
    - resetDate: Reset Date must be a number 1 - 31. Default to 1 in MONTHLY rotation.
    - randomizedPerRotation:
    - true: each goal will be randomly assigned to multiple periods
    - false: a goal will only be assigned to one period
    - tags: challenge's labels.

    Properties:
        url: /challenge/v1/admin/namespaces/{namespace}/challenges

        method: POST

        tags: ["Challenge Configuration"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelCreateChallengeRequest in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        201: Created - ModelChallengeResponse (Created)

        400: Bad Request - ResponseError (20018: bad request: {{message}})

        401: Unauthorized - IamErrorResponse (20001: unauthorized access)

        403: Forbidden - IamErrorResponse (20013: insufficient permission)

        409: Conflict - ResponseError (99002: duplicate key error: {{message}})

        422: Unprocessable Entity - ResponseError (99003: challenge validation error: {{message}})

        500: Internal Server Error - ResponseError (20000: internal server error: {{message}})
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminCreateChallenge.create(
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminDeleteChallenge)
def admin_delete_challenge(
    challenge_code: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete a Challenge (adminDeleteChallenge)

    - Required permission: ADMIN:NAMESPACE:{namespace}:CHALLENGE [DELETE]

    Properties:
        url: /challenge/v1/admin/namespaces/{namespace}/challenges/{challengeCode}

        method: DELETE

        tags: ["Challenge Configuration"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        challenge_code: (challengeCode) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (No Content)

        400: Bad Request - ResponseError (20018: bad request: {{message}})

        401: Unauthorized - ResponseError (20001: unauthorized access)

        403: Forbidden - ResponseError (20013: insufficient permission)

        404: Not Found - ResponseError (20029: not found)

        500: Internal Server Error - ResponseError (20000: internal server error: {{message}})
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminDeleteChallenge.create(
        challenge_code=challenge_code,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminDeleteChallenge)
async def admin_delete_challenge_async(
    challenge_code: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete a Challenge (adminDeleteChallenge)

    - Required permission: ADMIN:NAMESPACE:{namespace}:CHALLENGE [DELETE]

    Properties:
        url: /challenge/v1/admin/namespaces/{namespace}/challenges/{challengeCode}

        method: DELETE

        tags: ["Challenge Configuration"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        challenge_code: (challengeCode) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (No Content)

        400: Bad Request - ResponseError (20018: bad request: {{message}})

        401: Unauthorized - ResponseError (20001: unauthorized access)

        403: Forbidden - ResponseError (20013: insufficient permission)

        404: Not Found - ResponseError (20029: not found)

        500: Internal Server Error - ResponseError (20000: internal server error: {{message}})
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminDeleteChallenge.create(
        challenge_code=challenge_code,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminDeleteTiedChallenge)
def admin_delete_tied_challenge(
    challenge_code: str,
    safe_delete: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete Tied Challenge (adminDeleteTiedChallenge)

    - Required permission: ADMIN:NAMESPACE:{namespace}:CHALLENGE [DELETE]
    - This endpoint will delete the combination of related data: CHALLENGES, GOALS, SCHEDULES, PLAYER PROGRESSIONS

    Properties:
        url: /challenge/v1/admin/namespaces/{namespace}/challenges/{challengeCode}/tied

        method: DELETE

        tags: ["Challenge Configuration"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        challenge_code: (challengeCode) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        safe_delete: (safeDelete) OPTIONAL str in query

    Responses:
        204: No Content - (No Content)

        400: Bad Request - ResponseError (20018: bad request: {{message}})

        401: Unauthorized - ResponseError (20001: unauthorized access)

        403: Forbidden - ResponseError (20013: insufficient permission)

        404: Not Found - ResponseError (20029: not found)

        500: Internal Server Error - ResponseError (20000: internal server error: {{message}})
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminDeleteTiedChallenge.create(
        challenge_code=challenge_code,
        safe_delete=safe_delete,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminDeleteTiedChallenge)
async def admin_delete_tied_challenge_async(
    challenge_code: str,
    safe_delete: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete Tied Challenge (adminDeleteTiedChallenge)

    - Required permission: ADMIN:NAMESPACE:{namespace}:CHALLENGE [DELETE]
    - This endpoint will delete the combination of related data: CHALLENGES, GOALS, SCHEDULES, PLAYER PROGRESSIONS

    Properties:
        url: /challenge/v1/admin/namespaces/{namespace}/challenges/{challengeCode}/tied

        method: DELETE

        tags: ["Challenge Configuration"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        challenge_code: (challengeCode) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        safe_delete: (safeDelete) OPTIONAL str in query

    Responses:
        204: No Content - (No Content)

        400: Bad Request - ResponseError (20018: bad request: {{message}})

        401: Unauthorized - ResponseError (20001: unauthorized access)

        403: Forbidden - ResponseError (20013: insufficient permission)

        404: Not Found - ResponseError (20029: not found)

        500: Internal Server Error - ResponseError (20000: internal server error: {{message}})
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminDeleteTiedChallenge.create(
        challenge_code=challenge_code,
        safe_delete=safe_delete,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminGetActiveChallenges)
def admin_get_active_challenges(
    user_id: str,
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """List User's Active Challenges (adminGetActiveChallenges)

    - Required permission: ADMIN:NAMESPACE:{namespace}:CHALLENGE [READ]
    The endpoint provides you an access to list of currently active challenges in which the player is participating

    Properties:
        url: /challenge/v1/admin/namespaces/{namespace}/challenges/users/{userId}

        method: GET

        tags: ["Challenge Configuration"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

    Responses:
        200: OK - ModelListChallengeResponse (OK)

        401: Unauthorized - IamErrorResponse (20001: unauthorized access)

        403: Forbidden - IamErrorResponse (20013: insufficient permission)

        500: Internal Server Error - ResponseError (20000: internal server error: {{message}})
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminGetActiveChallenges.create(
        user_id=user_id,
        limit=limit,
        offset=offset,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminGetActiveChallenges)
async def admin_get_active_challenges_async(
    user_id: str,
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """List User's Active Challenges (adminGetActiveChallenges)

    - Required permission: ADMIN:NAMESPACE:{namespace}:CHALLENGE [READ]
    The endpoint provides you an access to list of currently active challenges in which the player is participating

    Properties:
        url: /challenge/v1/admin/namespaces/{namespace}/challenges/users/{userId}

        method: GET

        tags: ["Challenge Configuration"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

    Responses:
        200: OK - ModelListChallengeResponse (OK)

        401: Unauthorized - IamErrorResponse (20001: unauthorized access)

        403: Forbidden - IamErrorResponse (20013: insufficient permission)

        500: Internal Server Error - ResponseError (20000: internal server error: {{message}})
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminGetActiveChallenges.create(
        user_id=user_id,
        limit=limit,
        offset=offset,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminGetChallenge)
def admin_get_challenge(
    challenge_code: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get a Challenge (adminGetChallenge)

    - Required permission: ADMIN:NAMESPACE:{namespace}:CHALLENGE [READ]

    Properties:
        url: /challenge/v1/admin/namespaces/{namespace}/challenges/{challengeCode}

        method: GET

        tags: ["Challenge Configuration"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        challenge_code: (challengeCode) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ModelChallengeResponse (OK)

        401: Unauthorized - IamErrorResponse (20001: unauthorized access)

        403: Forbidden - IamErrorResponse (20013: insufficient permission)

        404: Not Found - ResponseError (20029: not found)

        500: Internal Server Error - ResponseError (20000: internal server error: {{message}})
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminGetChallenge.create(
        challenge_code=challenge_code,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminGetChallenge)
async def admin_get_challenge_async(
    challenge_code: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get a Challenge (adminGetChallenge)

    - Required permission: ADMIN:NAMESPACE:{namespace}:CHALLENGE [READ]

    Properties:
        url: /challenge/v1/admin/namespaces/{namespace}/challenges/{challengeCode}

        method: GET

        tags: ["Challenge Configuration"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        challenge_code: (challengeCode) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ModelChallengeResponse (OK)

        401: Unauthorized - IamErrorResponse (20001: unauthorized access)

        403: Forbidden - IamErrorResponse (20013: insufficient permission)

        404: Not Found - ResponseError (20029: not found)

        500: Internal Server Error - ResponseError (20000: internal server error: {{message}})
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminGetChallenge.create(
        challenge_code=challenge_code,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminGetChallenges)
def admin_get_challenges(
    keyword: Optional[str] = None,
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    sort_by: Optional[Union[str, AdminGetChallengesSortByEnum]] = None,
    status: Optional[Union[str, AdminGetChallengesStatusEnum]] = None,
    tags: Optional[List[str]] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """List Challenges (adminGetChallenges)

    - Required permission: ADMIN:NAMESPACE:{namespace}:CHALLENGE [READ]

    Properties:
        url: /challenge/v1/admin/namespaces/{namespace}/challenges

        method: GET

        tags: ["Challenge Configuration"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        keyword: (keyword) OPTIONAL str in query

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

        sort_by: (sortBy) OPTIONAL Union[str, SortByEnum] in query

        status: (status) OPTIONAL Union[str, StatusEnum] in query

        tags: (tags) OPTIONAL List[str] in query

    Responses:
        200: OK - ModelListChallengeResponse (OK)

        400: Bad Request - ResponseError (20018: bad request: {{message}})

        401: Unauthorized - IamErrorResponse (20001: unauthorized access)

        403: Forbidden - IamErrorResponse (20013: insufficient permission)

        500: Internal Server Error - ResponseError (20000: internal server error: {{message}})
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminGetChallenges.create(
        keyword=keyword,
        limit=limit,
        offset=offset,
        sort_by=sort_by,
        status=status,
        tags=tags,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminGetChallenges)
async def admin_get_challenges_async(
    keyword: Optional[str] = None,
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    sort_by: Optional[Union[str, AdminGetChallengesSortByEnum]] = None,
    status: Optional[Union[str, AdminGetChallengesStatusEnum]] = None,
    tags: Optional[List[str]] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """List Challenges (adminGetChallenges)

    - Required permission: ADMIN:NAMESPACE:{namespace}:CHALLENGE [READ]

    Properties:
        url: /challenge/v1/admin/namespaces/{namespace}/challenges

        method: GET

        tags: ["Challenge Configuration"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        keyword: (keyword) OPTIONAL str in query

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

        sort_by: (sortBy) OPTIONAL Union[str, SortByEnum] in query

        status: (status) OPTIONAL Union[str, StatusEnum] in query

        tags: (tags) OPTIONAL List[str] in query

    Responses:
        200: OK - ModelListChallengeResponse (OK)

        400: Bad Request - ResponseError (20018: bad request: {{message}})

        401: Unauthorized - IamErrorResponse (20001: unauthorized access)

        403: Forbidden - IamErrorResponse (20013: insufficient permission)

        500: Internal Server Error - ResponseError (20000: internal server error: {{message}})
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminGetChallenges.create(
        keyword=keyword,
        limit=limit,
        offset=offset,
        sort_by=sort_by,
        status=status,
        tags=tags,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminGetPeriods)
def admin_get_periods(
    challenge_code: str,
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get Challenge's Periods (adminGetPeriods)

    - Required permission: ADMIN:NAMESPACE:{namespace}:CHALLENGE [READ]

    Properties:
        url: /challenge/v1/admin/namespaces/{namespace}/challenges/{challengeCode}/periods

        method: GET

        tags: ["Challenge Configuration"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        challenge_code: (challengeCode) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

    Responses:
        200: OK - ModelListPeriodsResponse (OK)

        401: Unauthorized - IamErrorResponse (20001: unauthorized access)

        403: Forbidden - IamErrorResponse (20013: insufficient permission)

        404: Not Found - ResponseError (20029: not found)

        500: Internal Server Error - ResponseError (20000: internal server error: {{message}})
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminGetPeriods.create(
        challenge_code=challenge_code,
        limit=limit,
        offset=offset,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminGetPeriods)
async def admin_get_periods_async(
    challenge_code: str,
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get Challenge's Periods (adminGetPeriods)

    - Required permission: ADMIN:NAMESPACE:{namespace}:CHALLENGE [READ]

    Properties:
        url: /challenge/v1/admin/namespaces/{namespace}/challenges/{challengeCode}/periods

        method: GET

        tags: ["Challenge Configuration"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        challenge_code: (challengeCode) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

    Responses:
        200: OK - ModelListPeriodsResponse (OK)

        401: Unauthorized - IamErrorResponse (20001: unauthorized access)

        403: Forbidden - IamErrorResponse (20013: insufficient permission)

        404: Not Found - ResponseError (20029: not found)

        500: Internal Server Error - ResponseError (20000: internal server error: {{message}})
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminGetPeriods.create(
        challenge_code=challenge_code,
        limit=limit,
        offset=offset,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminRandomizeChallenge)
def admin_randomize_challenge(
    challenge_code: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Randomize Goals of a Challenge (adminRandomizeChallenge)

    - Required permission: ADMIN:NAMESPACE:{namespace}:CHALLENGE [UPDATE]
    This is a utility endpoint to execute randomize goals schedule on challenge that the assignmentRule is RANDOMIZED and RandomizePerRotation assigned with true.

    Properties:
        url: /challenge/v1/admin/namespaces/{namespace}/challenges/{challengeCode}/randomize

        method: POST

        tags: ["Challenge Configuration"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        challenge_code: (challengeCode) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - List[ModelSchedule] (OK)

        400: Bad Request - IamErrorResponse (20018: bad request: {{message}})

        401: Unauthorized - IamErrorResponse (20001: unauthorized access)

        403: Forbidden - IamErrorResponse (20013: insufficient permission)

        404: Not Found - ResponseError (20029: not found)

        500: Internal Server Error - ResponseError (20000: internal server error: {{message}})
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminRandomizeChallenge.create(
        challenge_code=challenge_code,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminRandomizeChallenge)
async def admin_randomize_challenge_async(
    challenge_code: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Randomize Goals of a Challenge (adminRandomizeChallenge)

    - Required permission: ADMIN:NAMESPACE:{namespace}:CHALLENGE [UPDATE]
    This is a utility endpoint to execute randomize goals schedule on challenge that the assignmentRule is RANDOMIZED and RandomizePerRotation assigned with true.

    Properties:
        url: /challenge/v1/admin/namespaces/{namespace}/challenges/{challengeCode}/randomize

        method: POST

        tags: ["Challenge Configuration"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        challenge_code: (challengeCode) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - List[ModelSchedule] (OK)

        400: Bad Request - IamErrorResponse (20018: bad request: {{message}})

        401: Unauthorized - IamErrorResponse (20001: unauthorized access)

        403: Forbidden - IamErrorResponse (20013: insufficient permission)

        404: Not Found - ResponseError (20029: not found)

        500: Internal Server Error - ResponseError (20000: internal server error: {{message}})
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminRandomizeChallenge.create(
        challenge_code=challenge_code,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminUpdateChallenge)
def admin_update_challenge(
    body: ModelUpdateChallengeRequest,
    challenge_code: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update a Challenge (adminUpdateChallenge)

    - Required permission: ADMIN:NAMESPACE:{namespace}:CHALLENGE [UPDATE]
    Request body:
    - name: name of the challenge
    - description: text describing about the challenge (optional)
    - startDate: timestamp of when the challenge is started
    - endDate: timestamp of when the challenge is ended (optional)
    - endAfter: describe number of period challenge will be retired after (optional). To configure challenge that never end, leave the endDate and endAfter field null/empty.
    - repeatAfter: describe number of period challenge's goals will be repeated after. Leave it empty if you don't want to repeat the challenge.
    - rotation: describe how long goals in a challenge will be available for players to progress before rotated with another goals. (DAILY|WEEKLY|MONTHLY|NONE)
    - activeGoalsPerRotation: number of goals per rotation (currently only applicable for RANDOMIZE assignment)
    - assignmentRule: describe how the goals will be assigned and scheduled to users. (FIXED|RANDOMIZED|UNSCHEDULED|CUSTOM)
    - goalsVisibility: describe whether users can see all goals under challenge, or only active goal in one rotation period only. (SHOWALL|PERIODONLY)
    - resetConfig: describe when rotation reset will happen (optional).
    - resetTime: Reset time must follow hours:minutes in 24 hours format (e.g. 01:30, 23:15) and in UTC timezone. Default to "00:00"
    - resetDay: Reset Day follows the ISO-8601 standard, from 1 (Monday) to 7 (Sunday). Default to 1 in WEEKLY rotation.
    - resetDate: Reset Date must be a number 1 - 31. Default to 1 in MONTHLY rotation.
    - randomizedPerRotation:
    - true: each goal will be randomly assigned to multiple periods
    - false: a goal will only be assigned to one period
    - tags: challenge's labels.

    Properties:
        url: /challenge/v1/admin/namespaces/{namespace}/challenges/{challengeCode}

        method: PUT

        tags: ["Challenge Configuration"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelUpdateChallengeRequest in body

        challenge_code: (challengeCode) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ModelChallengeResponse (OK)

        400: Bad Request - ResponseError (20018: bad request: {{message}})

        401: Unauthorized - ResponseError (Unauthorized)

        403: Forbidden - ResponseError (Forbidden)

        404: Not Found - ResponseError (Not Found)

        422: Unprocessable Entity - ResponseError (Unprocessable Entity)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminUpdateChallenge.create(
        body=body,
        challenge_code=challenge_code,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminUpdateChallenge)
async def admin_update_challenge_async(
    body: ModelUpdateChallengeRequest,
    challenge_code: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update a Challenge (adminUpdateChallenge)

    - Required permission: ADMIN:NAMESPACE:{namespace}:CHALLENGE [UPDATE]
    Request body:
    - name: name of the challenge
    - description: text describing about the challenge (optional)
    - startDate: timestamp of when the challenge is started
    - endDate: timestamp of when the challenge is ended (optional)
    - endAfter: describe number of period challenge will be retired after (optional). To configure challenge that never end, leave the endDate and endAfter field null/empty.
    - repeatAfter: describe number of period challenge's goals will be repeated after. Leave it empty if you don't want to repeat the challenge.
    - rotation: describe how long goals in a challenge will be available for players to progress before rotated with another goals. (DAILY|WEEKLY|MONTHLY|NONE)
    - activeGoalsPerRotation: number of goals per rotation (currently only applicable for RANDOMIZE assignment)
    - assignmentRule: describe how the goals will be assigned and scheduled to users. (FIXED|RANDOMIZED|UNSCHEDULED|CUSTOM)
    - goalsVisibility: describe whether users can see all goals under challenge, or only active goal in one rotation period only. (SHOWALL|PERIODONLY)
    - resetConfig: describe when rotation reset will happen (optional).
    - resetTime: Reset time must follow hours:minutes in 24 hours format (e.g. 01:30, 23:15) and in UTC timezone. Default to "00:00"
    - resetDay: Reset Day follows the ISO-8601 standard, from 1 (Monday) to 7 (Sunday). Default to 1 in WEEKLY rotation.
    - resetDate: Reset Date must be a number 1 - 31. Default to 1 in MONTHLY rotation.
    - randomizedPerRotation:
    - true: each goal will be randomly assigned to multiple periods
    - false: a goal will only be assigned to one period
    - tags: challenge's labels.

    Properties:
        url: /challenge/v1/admin/namespaces/{namespace}/challenges/{challengeCode}

        method: PUT

        tags: ["Challenge Configuration"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelUpdateChallengeRequest in body

        challenge_code: (challengeCode) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ModelChallengeResponse (OK)

        400: Bad Request - ResponseError (20018: bad request: {{message}})

        401: Unauthorized - ResponseError (Unauthorized)

        403: Forbidden - ResponseError (Forbidden)

        404: Not Found - ResponseError (Not Found)

        422: Unprocessable Entity - ResponseError (Unprocessable Entity)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminUpdateChallenge.create(
        body=body,
        challenge_code=challenge_code,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminUpdateTiedChallengeSchedule)
def admin_update_tied_challenge_schedule(
    body: ModelUpdateChallengeScheduleRequest,
    challenge_code: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update Tied Challenge Schedule (adminUpdateTiedChallengeSchedule)

    - Required permission: ADMIN:NAMESPACE:{namespace}:CHALLENGE [UPDATE]
    ### Request body:
    - action: Update the challenge schedule. The available options are:
    - STOP: Ends the challenge, changing its status to RETIRED. This option supports all types of challenges.
    - ACCELERATE: Speeds up the challenge's end time. Note that this option does not apply to challenges with an 'endAfter' value.
    - endDate: The timestamp specifying when the challenge should end (required if the action is ACCELERATE).

    Properties:
        url: /challenge/v1/admin/namespaces/{namespace}/challenges/{challengeCode}/tied/schedule

        method: PUT

        tags: ["Challenge Configuration"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelUpdateChallengeScheduleRequest in body

        challenge_code: (challengeCode) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ModelChallengeResponse (OK)

        400: Bad Request - ResponseError (20018: bad request: {{message}})

        401: Unauthorized - ResponseError (Unauthorized)

        403: Forbidden - ResponseError (Forbidden)

        404: Not Found - ResponseError (Not Found)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminUpdateTiedChallengeSchedule.create(
        body=body,
        challenge_code=challenge_code,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminUpdateTiedChallengeSchedule)
async def admin_update_tied_challenge_schedule_async(
    body: ModelUpdateChallengeScheduleRequest,
    challenge_code: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update Tied Challenge Schedule (adminUpdateTiedChallengeSchedule)

    - Required permission: ADMIN:NAMESPACE:{namespace}:CHALLENGE [UPDATE]
    ### Request body:
    - action: Update the challenge schedule. The available options are:
    - STOP: Ends the challenge, changing its status to RETIRED. This option supports all types of challenges.
    - ACCELERATE: Speeds up the challenge's end time. Note that this option does not apply to challenges with an 'endAfter' value.
    - endDate: The timestamp specifying when the challenge should end (required if the action is ACCELERATE).

    Properties:
        url: /challenge/v1/admin/namespaces/{namespace}/challenges/{challengeCode}/tied/schedule

        method: PUT

        tags: ["Challenge Configuration"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ModelUpdateChallengeScheduleRequest in body

        challenge_code: (challengeCode) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ModelChallengeResponse (OK)

        400: Bad Request - ResponseError (20018: bad request: {{message}})

        401: Unauthorized - ResponseError (Unauthorized)

        403: Forbidden - ResponseError (Forbidden)

        404: Not Found - ResponseError (Not Found)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminUpdateTiedChallengeSchedule.create(
        body=body,
        challenge_code=challenge_code,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



# Copyright (c) 2025 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.

from ._login import login
from ._login import login_async
from ._login import login_client
from ._login import login_client_async
from ._login import login_platform
from ._login import login_platform_async
from ._login import login_user
from ._login import login_user_async
from ._login import logout
from ._login import logout_async
from ._login import refresh_login
from ._login import refresh_login_async

from ._login_const import DEFAULT_AUTO_REFRESH
from ._login_const import DEFAULT_REFRESH_RATE

from ._jwt import parse_access_token

from ._utils import convert_bearer_auth_token_to_oauth_token_dict

from ._login import LoginTimerBase
from ._login import LoginClientTimer
from ._login import LoginPlatformTimer
from ._login import LoginUserTimer
from ._login import RefreshLoginTimer
from ._login import OnDemandTokenRefresher
from ._login import set_on_demand_token_refresher

__all__ = [
    # login
    "login",
    "login_async",
    "login_client",
    "login_client_async",
    "login_platform",
    "login_platform_async",
    "login_user",
    "login_user_async",
    "refresh_login",
    "refresh_login_async",
    "logout",
    "logout_async",
    # const
    "DEFAULT_AUTO_REFRESH",
    "DEFAULT_REFRESH_RATE",
    # jwt
    "parse_access_token",
    # utils
    "convert_bearer_auth_token_to_oauth_token_dict",
    # background
    "LoginTimerBase",
    "LoginClientTimer",
    "LoginPlatformTimer",
    "LoginUserTimer",
    "RefreshLoginTimer",
    # on-demand
    "OnDemandTokenRefresher",
    "set_on_demand_token_refresher",
]



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Challenge Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model
from ....core import StrEnum

from ..models.model_reset_config import ModelResetConfig


class AssignmentRuleEnum(StrEnum):
    CUSTOM = "CUSTOM"
    FIXED = "FIXED"
    RANDOMIZED = "RANDOMIZED"
    UNSCHEDULED = "UNSCHEDULED"


class GoalsVisibilityEnum(StrEnum):
    PERIODONLY = "PERIODONLY"
    SHOWALL = "SHOWALL"


class RotationEnum(StrEnum):
    DAILY = "DAILY"
    MONTHLY = "MONTHLY"
    NONE = "NONE"
    WEEKLY = "WEEKLY"


class ModelUpdateChallengeRequest(Model):
    """Model update challenge request (model.UpdateChallengeRequest)

    Properties:
        active_goals_per_rotation: (activeGoalsPerRotation) OPTIONAL int

        assignment_rule: (assignmentRule) OPTIONAL Union[str, AssignmentRuleEnum]

        description: (description) OPTIONAL str

        end_after: (endAfter) OPTIONAL int

        end_date: (endDate) OPTIONAL str

        goals_visibility: (goalsVisibility) OPTIONAL Union[str, GoalsVisibilityEnum]

        name: (name) OPTIONAL str

        randomized_per_rotation: (randomizedPerRotation) OPTIONAL bool

        repeat_after: (repeatAfter) OPTIONAL int

        reset_config: (resetConfig) OPTIONAL ModelResetConfig

        rotation: (rotation) OPTIONAL Union[str, RotationEnum]

        start_date: (startDate) OPTIONAL str

        tags: (tags) OPTIONAL List[str]
    """

    # region fields

    active_goals_per_rotation: int  # OPTIONAL
    assignment_rule: Union[str, AssignmentRuleEnum]  # OPTIONAL
    description: str  # OPTIONAL
    end_after: int  # OPTIONAL
    end_date: str  # OPTIONAL
    goals_visibility: Union[str, GoalsVisibilityEnum]  # OPTIONAL
    name: str  # OPTIONAL
    randomized_per_rotation: bool  # OPTIONAL
    repeat_after: int  # OPTIONAL
    reset_config: ModelResetConfig  # OPTIONAL
    rotation: Union[str, RotationEnum]  # OPTIONAL
    start_date: str  # OPTIONAL
    tags: List[str]  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_active_goals_per_rotation(self, value: int) -> ModelUpdateChallengeRequest:
        self.active_goals_per_rotation = value
        return self

    def with_assignment_rule(
        self, value: Union[str, AssignmentRuleEnum]
    ) -> ModelUpdateChallengeRequest:
        self.assignment_rule = value
        return self

    def with_description(self, value: str) -> ModelUpdateChallengeRequest:
        self.description = value
        return self

    def with_end_after(self, value: int) -> ModelUpdateChallengeRequest:
        self.end_after = value
        return self

    def with_end_date(self, value: str) -> ModelUpdateChallengeRequest:
        self.end_date = value
        return self

    def with_goals_visibility(
        self, value: Union[str, GoalsVisibilityEnum]
    ) -> ModelUpdateChallengeRequest:
        self.goals_visibility = value
        return self

    def with_name(self, value: str) -> ModelUpdateChallengeRequest:
        self.name = value
        return self

    def with_randomized_per_rotation(self, value: bool) -> ModelUpdateChallengeRequest:
        self.randomized_per_rotation = value
        return self

    def with_repeat_after(self, value: int) -> ModelUpdateChallengeRequest:
        self.repeat_after = value
        return self

    def with_reset_config(self, value: ModelResetConfig) -> ModelUpdateChallengeRequest:
        self.reset_config = value
        return self

    def with_rotation(
        self, value: Union[str, RotationEnum]
    ) -> ModelUpdateChallengeRequest:
        self.rotation = value
        return self

    def with_start_date(self, value: str) -> ModelUpdateChallengeRequest:
        self.start_date = value
        return self

    def with_tags(self, value: List[str]) -> ModelUpdateChallengeRequest:
        self.tags = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "active_goals_per_rotation"):
            result["activeGoalsPerRotation"] = int(self.active_goals_per_rotation)
        elif include_empty:
            result["activeGoalsPerRotation"] = 0
        if hasattr(self, "assignment_rule"):
            result["assignmentRule"] = str(self.assignment_rule)
        elif include_empty:
            result["assignmentRule"] = Union[str, AssignmentRuleEnum]()
        if hasattr(self, "description"):
            result["description"] = str(self.description)
        elif include_empty:
            result["description"] = ""
        if hasattr(self, "end_after"):
            result["endAfter"] = int(self.end_after)
        elif include_empty:
            result["endAfter"] = 0
        if hasattr(self, "end_date"):
            result["endDate"] = str(self.end_date)
        elif include_empty:
            result["endDate"] = ""
        if hasattr(self, "goals_visibility"):
            result["goalsVisibility"] = str(self.goals_visibility)
        elif include_empty:
            result["goalsVisibility"] = Union[str, GoalsVisibilityEnum]()
        if hasattr(self, "name"):
            result["name"] = str(self.name)
        elif include_empty:
            result["name"] = ""
        if hasattr(self, "randomized_per_rotation"):
            result["randomizedPerRotation"] = bool(self.randomized_per_rotation)
        elif include_empty:
            result["randomizedPerRotation"] = False
        if hasattr(self, "repeat_after"):
            result["repeatAfter"] = int(self.repeat_after)
        elif include_empty:
            result["repeatAfter"] = 0
        if hasattr(self, "reset_config"):
            result["resetConfig"] = self.reset_config.to_dict(
                include_empty=include_empty
            )
        elif include_empty:
            result["resetConfig"] = ModelResetConfig()
        if hasattr(self, "rotation"):
            result["rotation"] = str(self.rotation)
        elif include_empty:
            result["rotation"] = Union[str, RotationEnum]()
        if hasattr(self, "start_date"):
            result["startDate"] = str(self.start_date)
        elif include_empty:
            result["startDate"] = ""
        if hasattr(self, "tags"):
            result["tags"] = [str(i0) for i0 in self.tags]
        elif include_empty:
            result["tags"] = []
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        active_goals_per_rotation: Optional[int] = None,
        assignment_rule: Optional[Union[str, AssignmentRuleEnum]] = None,
        description: Optional[str] = None,
        end_after: Optional[int] = None,
        end_date: Optional[str] = None,
        goals_visibility: Optional[Union[str, GoalsVisibilityEnum]] = None,
        name: Optional[str] = None,
        randomized_per_rotation: Optional[bool] = None,
        repeat_after: Optional[int] = None,
        reset_config: Optional[ModelResetConfig] = None,
        rotation: Optional[Union[str, RotationEnum]] = None,
        start_date: Optional[str] = None,
        tags: Optional[List[str]] = None,
        **kwargs,
    ) -> ModelUpdateChallengeRequest:
        instance = cls()
        if active_goals_per_rotation is not None:
            instance.active_goals_per_rotation = active_goals_per_rotation
        if assignment_rule is not None:
            instance.assignment_rule = assignment_rule
        if description is not None:
            instance.description = description
        if end_after is not None:
            instance.end_after = end_after
        if end_date is not None:
            instance.end_date = end_date
        if goals_visibility is not None:
            instance.goals_visibility = goals_visibility
        if name is not None:
            instance.name = name
        if randomized_per_rotation is not None:
            instance.randomized_per_rotation = randomized_per_rotation
        if repeat_after is not None:
            instance.repeat_after = repeat_after
        if reset_config is not None:
            instance.reset_config = reset_config
        if rotation is not None:
            instance.rotation = rotation
        if start_date is not None:
            instance.start_date = start_date
        if tags is not None:
            instance.tags = tags
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ModelUpdateChallengeRequest:
        instance = cls()
        if not dict_:
            return instance
        if (
            "activeGoalsPerRotation" in dict_
            and dict_["activeGoalsPerRotation"] is not None
        ):
            instance.active_goals_per_rotation = int(dict_["activeGoalsPerRotation"])
        elif include_empty:
            instance.active_goals_per_rotation = 0
        if "assignmentRule" in dict_ and dict_["assignmentRule"] is not None:
            instance.assignment_rule = str(dict_["assignmentRule"])
        elif include_empty:
            instance.assignment_rule = Union[str, AssignmentRuleEnum]()
        if "description" in dict_ and dict_["description"] is not None:
            instance.description = str(dict_["description"])
        elif include_empty:
            instance.description = ""
        if "endAfter" in dict_ and dict_["endAfter"] is not None:
            instance.end_after = int(dict_["endAfter"])
        elif include_empty:
            instance.end_after = 0
        if "endDate" in dict_ and dict_["endDate"] is not None:
            instance.end_date = str(dict_["endDate"])
        elif include_empty:
            instance.end_date = ""
        if "goalsVisibility" in dict_ and dict_["goalsVisibility"] is not None:
            instance.goals_visibility = str(dict_["goalsVisibility"])
        elif include_empty:
            instance.goals_visibility = Union[str, GoalsVisibilityEnum]()
        if "name" in dict_ and dict_["name"] is not None:
            instance.name = str(dict_["name"])
        elif include_empty:
            instance.name = ""
        if (
            "randomizedPerRotation" in dict_
            and dict_["randomizedPerRotation"] is not None
        ):
            instance.randomized_per_rotation = bool(dict_["randomizedPerRotation"])
        elif include_empty:
            instance.randomized_per_rotation = False
        if "repeatAfter" in dict_ and dict_["repeatAfter"] is not None:
            instance.repeat_after = int(dict_["repeatAfter"])
        elif include_empty:
            instance.repeat_after = 0
        if "resetConfig" in dict_ and dict_["resetConfig"] is not None:
            instance.reset_config = ModelResetConfig.create_from_dict(
                dict_["resetConfig"], include_empty=include_empty
            )
        elif include_empty:
            instance.reset_config = ModelResetConfig()
        if "rotation" in dict_ and dict_["rotation"] is not None:
            instance.rotation = str(dict_["rotation"])
        elif include_empty:
            instance.rotation = Union[str, RotationEnum]()
        if "startDate" in dict_ and dict_["startDate"] is not None:
            instance.start_date = str(dict_["startDate"])
        elif include_empty:
            instance.start_date = ""
        if "tags" in dict_ and dict_["tags"] is not None:
            instance.tags = [str(i0) for i0 in dict_["tags"]]
        elif include_empty:
            instance.tags = []
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ModelUpdateChallengeRequest]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ModelUpdateChallengeRequest]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ModelUpdateChallengeRequest,
        List[ModelUpdateChallengeRequest],
        Dict[Any, ModelUpdateChallengeRequest],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "activeGoalsPerRotation": "active_goals_per_rotation",
            "assignmentRule": "assignment_rule",
            "description": "description",
            "endAfter": "end_after",
            "endDate": "end_date",
            "goalsVisibility": "goals_visibility",
            "name": "name",
            "randomizedPerRotation": "randomized_per_rotation",
            "repeatAfter": "repeat_after",
            "resetConfig": "reset_config",
            "rotation": "rotation",
            "startDate": "start_date",
            "tags": "tags",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "activeGoalsPerRotation": False,
            "assignmentRule": False,
            "description": False,
            "endAfter": False,
            "endDate": False,
            "goalsVisibility": False,
            "name": False,
            "randomizedPerRotation": False,
            "repeatAfter": False,
            "resetConfig": False,
            "rotation": False,
            "startDate": False,
            "tags": False,
        }

    @staticmethod
    def get_enum_map() -> Dict[str, List[Any]]:
        return {
            "assignmentRule": ["CUSTOM", "FIXED", "RANDOMIZED", "UNSCHEDULED"],
            "goalsVisibility": ["PERIODONLY", "SHOWALL"],
            "rotation": ["DAILY", "MONTHLY", "NONE", "WEEKLY"],
        }

    # endregion static methods



from typing import Optional

from ._ctypes import InsufficientPermissionsError
from ._ctypes import PermissionAction


class MockTokenValidator:
    def __init__(self, value: bool = False):
        self.value = value

    def validate_token(
        self,
        token: str,
        resource: Optional[str] = None,
        action: Optional[PermissionAction] = None,
        namespace: Optional[str] = None,
        user_id: Optional[str] = None,
        **kwargs,
    ) -> Optional[Exception]:
        return None if self.value else InsufficientPermissionsError()



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import HeaderStr
from ....core import get_namespace as get_services_namespace
from ....core import run_request
from ....core import run_request_async
from ....core import same_doc_as

from ..models import ModelsDataRetrievalResponse
from ..models import ModelsUserDataURL
from ..models import ModelsUserPersonalDataResponse
from ..models import ResponseError

from ..operations.data_retrieval_headless import PublicCancelMyPersonalDataRequest
from ..operations.data_retrieval_headless import PublicGenerateMyHeadlessPersonalDataURL
from ..operations.data_retrieval_headless import PublicGetMyPersonalDataRequests
from ..operations.data_retrieval_headless import (
    PublicSubmitMyHeadlessPersonalDataRequest,
)
from ..operations.data_retrieval_headless import PublicSubmitMyPersonalDataRequest


@same_doc_as(PublicCancelMyPersonalDataRequest)
def public_cancel_my_personal_data_request(
    request_date: str, x_additional_headers: Optional[Dict[str, str]] = None, **kwargs
):
    """Cancel my personal data request (PublicCancelMyPersonalDataRequest)

    Cancel a pending Personal Data Request by its request date. Requires a valid access token.

    Properties:
        url: /gdpr/public/users/me/requests/{requestDate}

        method: DELETE

        tags: ["Data Retrieval - Headless"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        request_date: (requestDate) REQUIRED str in path

    Responses:
        204: No Content - (No Content)

        400: Bad Request - ResponseError (Bad Request)

        401: Unauthorized - ResponseError (Unauthorized)

        403: Forbidden - ResponseError (Forbidden)

        404: Not Found - ResponseError (Not Found)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    request = PublicCancelMyPersonalDataRequest.create(
        request_date=request_date,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(PublicCancelMyPersonalDataRequest)
async def public_cancel_my_personal_data_request_async(
    request_date: str, x_additional_headers: Optional[Dict[str, str]] = None, **kwargs
):
    """Cancel my personal data request (PublicCancelMyPersonalDataRequest)

    Cancel a pending Personal Data Request by its request date. Requires a valid access token.

    Properties:
        url: /gdpr/public/users/me/requests/{requestDate}

        method: DELETE

        tags: ["Data Retrieval - Headless"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        request_date: (requestDate) REQUIRED str in path

    Responses:
        204: No Content - (No Content)

        400: Bad Request - ResponseError (Bad Request)

        401: Unauthorized - ResponseError (Unauthorized)

        403: Forbidden - ResponseError (Forbidden)

        404: Not Found - ResponseError (Not Found)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    request = PublicCancelMyPersonalDataRequest.create(
        request_date=request_date,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(PublicGenerateMyHeadlessPersonalDataURL)
def public_generate_my_headless_personal_data_url(
    request_date: str, x_additional_headers: Optional[Dict[str, str]] = None, **kwargs
):
    """Generate my headless personal data download url (PublicGenerateMyHeadlessPersonalDataURL)

    Generate a download url for a completed Personal Data Request for headless account scenario (without a password).
    **Typically used by web portal** to invoke headless player GDPR using 3rd platform web authentication.
    Requires a valid access token and a fresh **gdpr_token** cookie (max-age 5 minutes).
    The gdpr_token is issued by the IAM service during 3rd platform web authentication.

    If a full account is available, use `POST /gdpr/public/namespaces/{namespace}/users/{userId}/requests/{requestDate}/generate` instead.

    Properties:
        url: /gdpr/public/users/me/headless/requests/{requestDate}/generate

        method: POST

        tags: ["Data Retrieval - Headless"]

        consumes: ["*/*"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        request_date: (requestDate) REQUIRED str in path

    Responses:
        200: OK - ModelsUserDataURL (OK)

        400: Bad Request - ResponseError (Bad Request)

        401: Unauthorized - ResponseError (Unauthorized)

        403: Forbidden - ResponseError (Forbidden)

        404: Not Found - ResponseError (Not Found)

        429: Too Many Requests - ResponseError (Too Many Requests)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    request = PublicGenerateMyHeadlessPersonalDataURL.create(
        request_date=request_date,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(PublicGenerateMyHeadlessPersonalDataURL)
async def public_generate_my_headless_personal_data_url_async(
    request_date: str, x_additional_headers: Optional[Dict[str, str]] = None, **kwargs
):
    """Generate my headless personal data download url (PublicGenerateMyHeadlessPersonalDataURL)

    Generate a download url for a completed Personal Data Request for headless account scenario (without a password).
    **Typically used by web portal** to invoke headless player GDPR using 3rd platform web authentication.
    Requires a valid access token and a fresh **gdpr_token** cookie (max-age 5 minutes).
    The gdpr_token is issued by the IAM service during 3rd platform web authentication.

    If a full account is available, use `POST /gdpr/public/namespaces/{namespace}/users/{userId}/requests/{requestDate}/generate` instead.

    Properties:
        url: /gdpr/public/users/me/headless/requests/{requestDate}/generate

        method: POST

        tags: ["Data Retrieval - Headless"]

        consumes: ["*/*"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        request_date: (requestDate) REQUIRED str in path

    Responses:
        200: OK - ModelsUserDataURL (OK)

        400: Bad Request - ResponseError (Bad Request)

        401: Unauthorized - ResponseError (Unauthorized)

        403: Forbidden - ResponseError (Forbidden)

        404: Not Found - ResponseError (Not Found)

        429: Too Many Requests - ResponseError (Too Many Requests)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    request = PublicGenerateMyHeadlessPersonalDataURL.create(
        request_date=request_date,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(PublicGetMyPersonalDataRequests)
def public_get_my_personal_data_requests(
    x_additional_headers: Optional[Dict[str, str]] = None, **kwargs
):
    """Get my personal data requests (PublicGetMyPersonalDataRequests)

    List my personal data requests. Requires a valid access token.

    Properties:
        url: /gdpr/public/users/me/requests

        method: GET

        tags: ["Data Retrieval - Headless"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

    Responses:
        200: OK - ModelsUserPersonalDataResponse (OK)

        401: Unauthorized - ResponseError (Unauthorized)

        403: Forbidden - ResponseError (Forbidden)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    request = PublicGetMyPersonalDataRequests.create()
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(PublicGetMyPersonalDataRequests)
async def public_get_my_personal_data_requests_async(
    x_additional_headers: Optional[Dict[str, str]] = None, **kwargs
):
    """Get my personal data requests (PublicGetMyPersonalDataRequests)

    List my personal data requests. Requires a valid access token.

    Properties:
        url: /gdpr/public/users/me/requests

        method: GET

        tags: ["Data Retrieval - Headless"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

    Responses:
        200: OK - ModelsUserPersonalDataResponse (OK)

        401: Unauthorized - ResponseError (Unauthorized)

        403: Forbidden - ResponseError (Forbidden)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    request = PublicGetMyPersonalDataRequests.create()
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(PublicSubmitMyHeadlessPersonalDataRequest)
def public_submit_my_headless_personal_data_request(
    email: Optional[str] = None,
    language_tag: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Submit my headless personal data request (PublicSubmitMyHeadlessPersonalDataRequest)

    Submit a Personal Data Request for headless account scenario (without a password).
    **Typically used by web portal** to invoke headless player GDPR using 3rd platform web authentication.
    Requires a valid access token and a fresh **gdpr_token** cookie (max-age 5 minutes).
    The gdpr_token is issued by the IAM service during 3rd platform web authentication.

    If a full account is available, use `POST /gdpr/public/namespaces/{namespace}/users/{userId}/requests` instead.

    ### Request Header:
    - **Content-Type: application/x-www-form-urlencoded**

    Properties:
        url: /gdpr/public/users/me/headless/requests

        method: POST

        tags: ["Data Retrieval - Headless"]

        consumes: ["application/x-www-form-urlencoded"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        email: (email) OPTIONAL str in form_data

        language_tag: (languageTag) OPTIONAL str in form_data

    Responses:
        201: Created - ModelsDataRetrievalResponse (Created)

        400: Bad Request - ResponseError (Bad Request)

        401: Unauthorized - ResponseError (Unauthorized)

        403: Forbidden - ResponseError (Forbidden)

        429: Too Many Requests - ResponseError (Too Many Requests)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    request = PublicSubmitMyHeadlessPersonalDataRequest.create(
        email=email,
        language_tag=language_tag,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(PublicSubmitMyHeadlessPersonalDataRequest)
async def public_submit_my_headless_personal_data_request_async(
    email: Optional[str] = None,
    language_tag: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Submit my headless personal data request (PublicSubmitMyHeadlessPersonalDataRequest)

    Submit a Personal Data Request for headless account scenario (without a password).
    **Typically used by web portal** to invoke headless player GDPR using 3rd platform web authentication.
    Requires a valid access token and a fresh **gdpr_token** cookie (max-age 5 minutes).
    The gdpr_token is issued by the IAM service during 3rd platform web authentication.

    If a full account is available, use `POST /gdpr/public/namespaces/{namespace}/users/{userId}/requests` instead.

    ### Request Header:
    - **Content-Type: application/x-www-form-urlencoded**

    Properties:
        url: /gdpr/public/users/me/headless/requests

        method: POST

        tags: ["Data Retrieval - Headless"]

        consumes: ["application/x-www-form-urlencoded"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        email: (email) OPTIONAL str in form_data

        language_tag: (languageTag) OPTIONAL str in form_data

    Responses:
        201: Created - ModelsDataRetrievalResponse (Created)

        400: Bad Request - ResponseError (Bad Request)

        401: Unauthorized - ResponseError (Unauthorized)

        403: Forbidden - ResponseError (Forbidden)

        429: Too Many Requests - ResponseError (Too Many Requests)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    request = PublicSubmitMyHeadlessPersonalDataRequest.create(
        email=email,
        language_tag=language_tag,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(PublicSubmitMyPersonalDataRequest)
def public_submit_my_personal_data_request(
    platform_id: str,
    platform_token: str,
    email: Optional[str] = None,
    language_tag: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Submit my personal data request (PublicSubmitMyPersonalDataRequest)

    Submit a Personal Data Request authenticated by a platform token.
    Requires a valid access token.

    This endpoint is intended for **in-game use** only and requires a valid platformId and platform token.
    If a full account is available, use `POST /gdpr/public/namespaces/{namespace}/users/{userId}/requests` instead.

    ### Request Header:
    - **Content-Type: application/x-www-form-urlencoded**

    Properties:
        url: /gdpr/public/users/me/requests

        method: POST

        tags: ["Data Retrieval - Headless"]

        consumes: ["application/x-www-form-urlencoded"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        email: (email) OPTIONAL str in form_data

        language_tag: (languageTag) OPTIONAL str in form_data

        platform_id: (platformId) REQUIRED str in form_data

        platform_token: (platformToken) REQUIRED str in form_data

    Responses:
        201: Created - ModelsDataRetrievalResponse (Created)

        400: Bad Request - ResponseError (Bad Request)

        401: Unauthorized - ResponseError (Unauthorized)

        403: Forbidden - ResponseError (Forbidden)

        429: Too Many Requests - ResponseError (Too Many Requests)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    request = PublicSubmitMyPersonalDataRequest.create(
        platform_id=platform_id,
        platform_token=platform_token,
        email=email,
        language_tag=language_tag,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(PublicSubmitMyPersonalDataRequest)
async def public_submit_my_personal_data_request_async(
    platform_id: str,
    platform_token: str,
    email: Optional[str] = None,
    language_tag: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Submit my personal data request (PublicSubmitMyPersonalDataRequest)

    Submit a Personal Data Request authenticated by a platform token.
    Requires a valid access token.

    This endpoint is intended for **in-game use** only and requires a valid platformId and platform token.
    If a full account is available, use `POST /gdpr/public/namespaces/{namespace}/users/{userId}/requests` instead.

    ### Request Header:
    - **Content-Type: application/x-www-form-urlencoded**

    Properties:
        url: /gdpr/public/users/me/requests

        method: POST

        tags: ["Data Retrieval - Headless"]

        consumes: ["application/x-www-form-urlencoded"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        email: (email) OPTIONAL str in form_data

        language_tag: (languageTag) OPTIONAL str in form_data

        platform_id: (platformId) REQUIRED str in form_data

        platform_token: (platformToken) REQUIRED str in form_data

    Responses:
        201: Created - ModelsDataRetrievalResponse (Created)

        400: Bad Request - ResponseError (Bad Request)

        401: Unauthorized - ResponseError (Unauthorized)

        403: Forbidden - ResponseError (Forbidden)

        429: Too Many Requests - ResponseError (Too Many Requests)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    request = PublicSubmitMyPersonalDataRequest.create(
        platform_id=platform_id,
        platform_token=platform_token,
        email=email,
        language_tag=language_tag,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import HeaderStr
from ....core import get_namespace as get_services_namespace
from ....core import run_request
from ....core import run_request_async
from ....core import same_doc_as

from ..models import ClientmodelListClientPermissionSet
from ..models import ClientmodelListTemplatesResponse
from ..models import ClientmodelListUpsertModulesRequest
from ..models import ClientmodelPermissionSetDeleteGroupRequest
from ..models import ClientmodelUpsertPermissionModulePackageRequest
from ..models import RestErrorResponse

from ..operations.clients_config_v3 import AdminDeleteConfigPermissionsByGroup
from ..operations.clients_config_v3 import AdminListClientAvailablePermissions
from ..operations.clients_config_v3 import AdminListClientTemplates
from ..operations.clients_config_v3 import AdminUpdateAvailablePermissionsByModule
from ..operations.clients_config_v3 import AdminUpdateModulePackage


@same_doc_as(AdminDeleteConfigPermissionsByGroup)
def admin_delete_config_permissions_by_group(
    body: ClientmodelPermissionSetDeleteGroupRequest,
    force_delete: Optional[bool] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete Client config permissions by module and group (AdminDeleteConfigPermissionsByGroup)

    Delete Client config permissions by module and group.

    Properties:
        url: /iam/v3/admin/clientConfig/permissions

        method: DELETE

        tags: ["Clients Config V3"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ClientmodelPermissionSetDeleteGroupRequest in body

        force_delete: (forceDelete) OPTIONAL bool in query

    Responses:
        204: No Content - (Operation succeeded)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)
    """
    request = AdminDeleteConfigPermissionsByGroup.create(
        body=body,
        force_delete=force_delete,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminDeleteConfigPermissionsByGroup)
async def admin_delete_config_permissions_by_group_async(
    body: ClientmodelPermissionSetDeleteGroupRequest,
    force_delete: Optional[bool] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete Client config permissions by module and group (AdminDeleteConfigPermissionsByGroup)

    Delete Client config permissions by module and group.

    Properties:
        url: /iam/v3/admin/clientConfig/permissions

        method: DELETE

        tags: ["Clients Config V3"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ClientmodelPermissionSetDeleteGroupRequest in body

        force_delete: (forceDelete) OPTIONAL bool in query

    Responses:
        204: No Content - (Operation succeeded)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)
    """
    request = AdminDeleteConfigPermissionsByGroup.create(
        body=body,
        force_delete=force_delete,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminListClientAvailablePermissions)
def admin_list_client_available_permissions(
    exclude_permissions: Optional[bool] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """List Client available permissions (AdminListClientAvailablePermissions)

    Lists all available client permissions.

    Properties:
        url: /iam/v3/admin/clientConfig/permissions

        method: GET

        tags: ["Clients Config V3"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        exclude_permissions: (excludePermissions) OPTIONAL bool in query

    Responses:
        200: OK - ClientmodelListClientPermissionSet (OK)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)
    """
    request = AdminListClientAvailablePermissions.create(
        exclude_permissions=exclude_permissions,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminListClientAvailablePermissions)
async def admin_list_client_available_permissions_async(
    exclude_permissions: Optional[bool] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """List Client available permissions (AdminListClientAvailablePermissions)

    Lists all available client permissions.

    Properties:
        url: /iam/v3/admin/clientConfig/permissions

        method: GET

        tags: ["Clients Config V3"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        exclude_permissions: (excludePermissions) OPTIONAL bool in query

    Responses:
        200: OK - ClientmodelListClientPermissionSet (OK)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)
    """
    request = AdminListClientAvailablePermissions.create(
        exclude_permissions=exclude_permissions,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminListClientTemplates)
def admin_list_client_templates(
    x_additional_headers: Optional[Dict[str, str]] = None, **kwargs
):
    """List Client templates (AdminListClientTemplates)

    Lists all available client templates.

    Properties:
        url: /iam/v3/admin/clientConfig/templates

        method: GET

        tags: ["Clients Config V3"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

    Responses:
        200: OK - ClientmodelListTemplatesResponse (OK)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)
    """
    request = AdminListClientTemplates.create()
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminListClientTemplates)
async def admin_list_client_templates_async(
    x_additional_headers: Optional[Dict[str, str]] = None, **kwargs
):
    """List Client templates (AdminListClientTemplates)

    Lists all available client templates.

    Properties:
        url: /iam/v3/admin/clientConfig/templates

        method: GET

        tags: ["Clients Config V3"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

    Responses:
        200: OK - ClientmodelListTemplatesResponse (OK)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)
    """
    request = AdminListClientTemplates.create()
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminUpdateAvailablePermissionsByModule)
def admin_update_available_permissions_by_module(
    body: ClientmodelListUpsertModulesRequest,
    force_delete: Optional[bool] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update or create Client permissions module (AdminUpdateAvailablePermissionsByModule)

    Upserts client available permissions at the module level.
    Only the modules listed in the request are processed - any other existing module is left untouched, so *callers do not need to include unchanged modules*.

    ## Groups update behavior

    For each module in the request, the 'groups' array is the *full desired state* of that module's groups - any existing group whose 'groupId' is omitted is treated as ***removed***.
    If a removed group is currently used by any client, the request will fails with 400 and nothing is saved, unless 'forceDelete=true' query param is passed.

    Properties:
        url: /iam/v3/admin/clientConfig/permissions

        method: PUT

        tags: ["Clients Config V3"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ClientmodelListUpsertModulesRequest in body

        force_delete: (forceDelete) OPTIONAL bool in query

    Responses:
        204: No Content - (Operation succeeded)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)
    """
    request = AdminUpdateAvailablePermissionsByModule.create(
        body=body,
        force_delete=force_delete,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminUpdateAvailablePermissionsByModule)
async def admin_update_available_permissions_by_module_async(
    body: ClientmodelListUpsertModulesRequest,
    force_delete: Optional[bool] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update or create Client permissions module (AdminUpdateAvailablePermissionsByModule)

    Upserts client available permissions at the module level.
    Only the modules listed in the request are processed - any other existing module is left untouched, so *callers do not need to include unchanged modules*.

    ## Groups update behavior

    For each module in the request, the 'groups' array is the *full desired state* of that module's groups - any existing group whose 'groupId' is omitted is treated as ***removed***.
    If a removed group is currently used by any client, the request will fails with 400 and nothing is saved, unless 'forceDelete=true' query param is passed.

    Properties:
        url: /iam/v3/admin/clientConfig/permissions

        method: PUT

        tags: ["Clients Config V3"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ClientmodelListUpsertModulesRequest in body

        force_delete: (forceDelete) OPTIONAL bool in query

    Responses:
        204: No Content - (Operation succeeded)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)
    """
    request = AdminUpdateAvailablePermissionsByModule.create(
        body=body,
        force_delete=force_delete,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminUpdateModulePackage)
def admin_update_module_package(
    body: ClientmodelUpsertPermissionModulePackageRequest,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Set permission module package info (AdminUpdateModulePackage)

    Sets the package info for a permission module. Ignores the module if it is not found.

    Properties:
        url: /iam/v3/admin/clientConfig/permissions/package

        method: PATCH

        tags: ["Clients Config V3"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ClientmodelUpsertPermissionModulePackageRequest in body

    Responses:
        204: No Content - (Operation succeeded)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)
    """
    request = AdminUpdateModulePackage.create(
        body=body,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminUpdateModulePackage)
async def admin_update_module_package_async(
    body: ClientmodelUpsertPermissionModulePackageRequest,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Set permission module package info (AdminUpdateModulePackage)

    Sets the package info for a permission module. Ignores the module if it is not found.

    Properties:
        url: /iam/v3/admin/clientConfig/permissions/package

        method: PATCH

        tags: ["Clients Config V3"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ClientmodelUpsertPermissionModulePackageRequest in body

    Responses:
        204: No Content - (Operation succeeded)

        401: Unauthorized - RestErrorResponse (20001: unauthorized access)

        403: Forbidden - RestErrorResponse (20013: insufficient permissions)
    """
    request = AdminUpdateModulePackage.create(
        body=body,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

"""Auto-generated package that contains models used by the AccelByte Gaming Services Gdpr Service."""

__version__ = "2.24.0"
__author__ = "AccelByte"
__email__ = "dev@accelbyte.net"

# pylint: disable=line-too-long

from .dto_extend_config_dto import DtoExtendConfigDTO
from .dto_finished_data_deletion import DtoFinishedDataDeletion
from .dto_finished_data_deletion import StatusEnum as DtoFinishedDataDeletionStatusEnum
from .dto_finished_data_request import DtoFinishedDataRequest
from .dto_finished_data_request import StatusEnum as DtoFinishedDataRequestStatusEnum
from .dto_list_finished_data_deletion import DtoListFinishedDataDeletion
from .dto_list_finished_data_requests import DtoListFinishedDataRequests
from .dto_platform_account_closure_client_request import (
    DtoPlatformAccountClosureClientRequest,
)
from .dto_platform_account_closure_client_response import (
    DtoPlatformAccountClosureClientResponse,
)
from .dto_platform_account_closure_clients_response import (
    DtoPlatformAccountClosureClientsResponse,
)
from .dto_platform_account_closure_mock_request import (
    DtoPlatformAccountClosureMockRequest,
)
from .dto_s2s_data_request_summary import DtoS2SDataRequestSummary
from .dto_s2s_data_request_summary import (
    StatusEnum as DtoS2SDataRequestSummaryStatusEnum,
)
from .dto_service_config_dto import DtoServiceConfigDTO
from .dto_service_config_dto import ProtocolEnum as DtoServiceConfigDTOProtocolEnum
from .dto_service_configuration_dto import DtoServiceConfigurationDTO
from .dto_service_configuration_dto import (
    TypeEnum as DtoServiceConfigurationDTOTypeEnum,
)
from .dto_service_configuration_update_request import (
    DtoServiceConfigurationUpdateRequest,
)
from .dto_services_configuration_response import DtoServicesConfigurationResponse
from .dto_user_platform_account_closure_histories_response import (
    DtoUserPlatformAccountClosureHistoriesResponse,
)
from .dto_user_platform_account_closure_history import (
    DtoUserPlatformAccountClosureHistory,
)
from .dto_xbox_bp_cert_validation_request import DtoXboxBPCertValidationRequest
from .dto_xbox_bp_cert_validation_response import DtoXboxBPCertValidationResponse
from .models_data_retrieval_response import ModelsDataRetrievalResponse
from .models_deletion_data import ModelsDeletionData
from .models_deletion_status import ModelsDeletionStatus
from .models_list_deletion_data_response import ModelsListDeletionDataResponse
from .models_list_personal_data_response import ModelsListPersonalDataResponse
from .models_pagination import ModelsPagination
from .models_personal_data import ModelsPersonalData
from .models_request_delete_response import ModelsRequestDeleteResponse
from .models_s2s_data_retrieval_response import ModelsS2SDataRetrievalResponse
from .models_s2s_request_delete_response import ModelsS2SRequestDeleteResponse
from .models_s2s_user_data_url import ModelsS2SUserDataURL
from .models_user_data_url import ModelsUserDataURL
from .models_user_personal_data import ModelsUserPersonalData
from .models_user_personal_data_response import ModelsUserPersonalDataResponse
from .response_error import ResponseError



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# Fleet Commander

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model

from ..models.api_fleet_bulk_action_error_item_response import (
    ApiFleetBulkActionErrorItemResponse,
)


class ApiFleetBulkDeleteResponse(Model):
    """Api fleet bulk delete response (api.FleetBulkDeleteResponse)

    Properties:
        errors: (errors) REQUIRED List[ApiFleetBulkActionErrorItemResponse]

        success_count: (successCount) REQUIRED int

        total_count: (totalCount) REQUIRED int
    """

    # region fields

    errors: List[ApiFleetBulkActionErrorItemResponse]  # REQUIRED
    success_count: int  # REQUIRED
    total_count: int  # REQUIRED

    # endregion fields

    # region with_x methods

    def with_errors(
        self, value: List[ApiFleetBulkActionErrorItemResponse]
    ) -> ApiFleetBulkDeleteResponse:
        self.errors = value
        return self

    def with_success_count(self, value: int) -> ApiFleetBulkDeleteResponse:
        self.success_count = value
        return self

    def with_total_count(self, value: int) -> ApiFleetBulkDeleteResponse:
        self.total_count = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "errors"):
            result["errors"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.errors
            ]
        elif include_empty:
            result["errors"] = []
        if hasattr(self, "success_count"):
            result["successCount"] = int(self.success_count)
        elif include_empty:
            result["successCount"] = 0
        if hasattr(self, "total_count"):
            result["totalCount"] = int(self.total_count)
        elif include_empty:
            result["totalCount"] = 0
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        errors: List[ApiFleetBulkActionErrorItemResponse],
        success_count: int,
        total_count: int,
        **kwargs,
    ) -> ApiFleetBulkDeleteResponse:
        instance = cls()
        instance.errors = errors
        instance.success_count = success_count
        instance.total_count = total_count
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ApiFleetBulkDeleteResponse:
        instance = cls()
        if not dict_:
            return instance
        if "errors" in dict_ and dict_["errors"] is not None:
            instance.errors = [
                ApiFleetBulkActionErrorItemResponse.create_from_dict(
                    i0, include_empty=include_empty
                )
                for i0 in dict_["errors"]
            ]
        elif include_empty:
            instance.errors = []
        if "successCount" in dict_ and dict_["successCount"] is not None:
            instance.success_count = int(dict_["successCount"])
        elif include_empty:
            instance.success_count = 0
        if "totalCount" in dict_ and dict_["totalCount"] is not None:
            instance.total_count = int(dict_["totalCount"])
        elif include_empty:
            instance.total_count = 0
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ApiFleetBulkDeleteResponse]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ApiFleetBulkDeleteResponse]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ApiFleetBulkDeleteResponse,
        List[ApiFleetBulkDeleteResponse],
        Dict[Any, ApiFleetBulkDeleteResponse],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "errors": "errors",
            "successCount": "success_count",
            "totalCount": "total_count",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "errors": True,
            "successCount": True,
            "totalCount": True,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

"""Auto-generated package that contains models used by the Custom Service Manager."""

__version__ = "1.34.0"
__author__ = "AccelByte"
__email__ = "dev@accelbyte.net"

# pylint: disable=line-too-long

from .create_new_no_sql_datab_b22b67 import CreateNewNoSQLDatabaseCredentialV2
from .create_no_sql_cluster_v2 import CreateNoSQLClusterV2
from .create_no_sql_database__189b02 import CreateNoSQLDatabaseCredentialV2
from .create_no_sql_database_v2 import CreateNoSQLDatabaseV2
from .delete_no_sql_cluster_v2 import DeleteNoSQLClusterV2
from .delete_no_sql_database_v2 import DeleteNoSQLDatabaseV2
from .get_no_sql_access_tunnel_v2 import GetNoSQLAccessTunnelV2
from .get_no_sql_app_list_v2 import GetNoSQLAppListV2
from .get_no_sql_cluster_v2 import GetNoSQLClusterV2
from .get_no_sql_database_v2 import GetNoSQLDatabaseV2
from .start_no_sql_cluster_v2 import StartNoSQLClusterV2
from .stop_no_sql_cluster_v2 import StopNoSQLClusterV2
from .update_no_sql_cluster_v2 import UpdateNoSQLClusterV2



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# Custom Service Manager

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model


class GeneratedGetDeploymentV1Response(Model):
    """Generated get deployment V1 response (generated.GetDeploymentV1Response)

    Properties:
        deployment_id: (deploymentId) REQUIRED str

        deleted_at: (deletedAt) OPTIONAL str

        image_tag: (imageTag) OPTIONAL str

        message: (message) OPTIONAL str

        status: (status) OPTIONAL str

        updated_at: (updatedAt) OPTIONAL str
    """

    # region fields

    deployment_id: str  # REQUIRED
    deleted_at: str  # OPTIONAL
    image_tag: str  # OPTIONAL
    message: str  # OPTIONAL
    status: str  # OPTIONAL
    updated_at: str  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_deployment_id(self, value: str) -> GeneratedGetDeploymentV1Response:
        self.deployment_id = value
        return self

    def with_deleted_at(self, value: str) -> GeneratedGetDeploymentV1Response:
        self.deleted_at = value
        return self

    def with_image_tag(self, value: str) -> GeneratedGetDeploymentV1Response:
        self.image_tag = value
        return self

    def with_message(self, value: str) -> GeneratedGetDeploymentV1Response:
        self.message = value
        return self

    def with_status(self, value: str) -> GeneratedGetDeploymentV1Response:
        self.status = value
        return self

    def with_updated_at(self, value: str) -> GeneratedGetDeploymentV1Response:
        self.updated_at = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "deployment_id"):
            result["deploymentId"] = str(self.deployment_id)
        elif include_empty:
            result["deploymentId"] = ""
        if hasattr(self, "deleted_at"):
            result["deletedAt"] = str(self.deleted_at)
        elif include_empty:
            result["deletedAt"] = ""
        if hasattr(self, "image_tag"):
            result["imageTag"] = str(self.image_tag)
        elif include_empty:
            result["imageTag"] = ""
        if hasattr(self, "message"):
            result["message"] = str(self.message)
        elif include_empty:
            result["message"] = ""
        if hasattr(self, "status"):
            result["status"] = str(self.status)
        elif include_empty:
            result["status"] = ""
        if hasattr(self, "updated_at"):
            result["updatedAt"] = str(self.updated_at)
        elif include_empty:
            result["updatedAt"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        deployment_id: str,
        deleted_at: Optional[str] = None,
        image_tag: Optional[str] = None,
        message: Optional[str] = None,
        status: Optional[str] = None,
        updated_at: Optional[str] = None,
        **kwargs,
    ) -> GeneratedGetDeploymentV1Response:
        instance = cls()
        instance.deployment_id = deployment_id
        if deleted_at is not None:
            instance.deleted_at = deleted_at
        if image_tag is not None:
            instance.image_tag = image_tag
        if message is not None:
            instance.message = message
        if status is not None:
            instance.status = status
        if updated_at is not None:
            instance.updated_at = updated_at
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> GeneratedGetDeploymentV1Response:
        instance = cls()
        if not dict_:
            return instance
        if "deploymentId" in dict_ and dict_["deploymentId"] is not None:
            instance.deployment_id = str(dict_["deploymentId"])
        elif include_empty:
            instance.deployment_id = ""
        if "deletedAt" in dict_ and dict_["deletedAt"] is not None:
            instance.deleted_at = str(dict_["deletedAt"])
        elif include_empty:
            instance.deleted_at = ""
        if "imageTag" in dict_ and dict_["imageTag"] is not None:
            instance.image_tag = str(dict_["imageTag"])
        elif include_empty:
            instance.image_tag = ""
        if "message" in dict_ and dict_["message"] is not None:
            instance.message = str(dict_["message"])
        elif include_empty:
            instance.message = ""
        if "status" in dict_ and dict_["status"] is not None:
            instance.status = str(dict_["status"])
        elif include_empty:
            instance.status = ""
        if "updatedAt" in dict_ and dict_["updatedAt"] is not None:
            instance.updated_at = str(dict_["updatedAt"])
        elif include_empty:
            instance.updated_at = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, GeneratedGetDeploymentV1Response]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[GeneratedGetDeploymentV1Response]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        GeneratedGetDeploymentV1Response,
        List[GeneratedGetDeploymentV1Response],
        Dict[Any, GeneratedGetDeploymentV1Response],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "deploymentId": "deployment_id",
            "deletedAt": "deleted_at",
            "imageTag": "image_tag",
            "message": "message",
            "status": "status",
            "updatedAt": "updated_at",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "deploymentId": True,
            "deletedAt": False,
            "imageTag": False,
            "message": False,
            "status": False,
            "updatedAt": False,
        }

    # endregion static methods
